@@ -66,32 +66,26 @@ import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy as L
 import qualified Data.Serialize
 import Data.Text (Text, pack)
-import Data.Monoid (mempty)
 import qualified Data.Map as M
 import Data.IORef (readIORef, newIORef, writeIORef)
-import Network.Wai (pathInfo, rawPathInfo, responseLBS)
+import Network.Wai (pathInfo)
 import Data.Char (isLower, isDigit)
 import Data.List (foldl')
 import qualified Data.ByteString as S
-import Network.HTTP.Types (status301)
 import System.PosixCompat.Files (getFileStatus, modificationTime)
 import System.Posix.Types (EpochTime)
 import Data.Conduit (($$))
 import Data.Conduit.List (sourceList)
 import Data.Functor.Identity (runIdentity)
+import qualified Filesystem.Path.CurrentOS as F
 
 import Network.Wai.Application.Static
     ( StaticSettings (..)
-    , defaultWebAppSettings
     , staticApp
-    , embeddedLookup
-    , toEmbedded
-    , toFilePath
-    , fromFilePath
-    , FilePath
-    , ETagLookup
     , webAppSettingsWithLookup
+    , embeddedSettings
     )
+import WaiAppStatic.Storage.Filesystem (ETagLookup)
 
 -- | Type used for the subsite with static contents.
 newtype Static = Static StaticSettings
@@ -107,7 +101,7 @@ type StaticRoute = Route Static
 static :: Prelude.FilePath -> IO Static
 static dir = do
     hashLookup <- cachedETagLookup dir
-    return $ Static $ webAppSettingsWithLookup (toFilePath dir) hashLookup
+    return $ Static $ webAppSettingsWithLookup (F.decodeString dir) hashLookup
 
 -- | Same as 'static', but does not assumes that the files do not
 -- change and checks their modification time whenever a request
@@ -115,15 +109,19 @@ static dir = do
 staticDevel :: Prelude.FilePath -> IO Static
 staticDevel dir = do
     hashLookup <- cachedETagLookupDevel dir
-    return $ Static $ webAppSettingsWithLookup (toFilePath dir) hashLookup
+    return $ Static $ webAppSettingsWithLookup (F.decodeString dir) hashLookup
 
 -- | Produce a 'Static' based on embedding all of the static
 -- files' contents in the executable at compile time.
+-- Nota Bene: if you replace the scaffolded 'static' call in Settings/StaticFiles.hs
+-- you will need to change the scaffolded addStaticContent.  Otherwise, some of your
+-- assets will be 404'ed.  This is because by default yesod will generate compile those
+-- assets to @static/tmp@ which for 'static' is fine since they are served out of the 
+-- directory itself.  With embedded static, that will not work.  
+-- You can easily change @addStaticContent@ to @\_ _ _ -> return Nothing@ as a workaround.
+-- This will cause yesod to embed those assets into the generated HTML file itself.  
 embed :: Prelude.FilePath -> Q Exp
-embed fp =
-    [|Static (defaultWebAppSettings
-        { ssFolder = embeddedLookup (toEmbedded $(embedDir fp))
-        })|]
+embed fp = [|Static (embeddedSettings $(embedDir fp))|]
 
 instance RenderRoute Static where
     -- | A route on the static subsite (see also 'staticFiles').
@@ -145,11 +143,7 @@ instance RenderRoute Static where
     renderRoute (StaticRoute x y) = (x, y)
 
 instance Yesod master => YesodDispatch Static master where
-    -- Need to append trailing slash to make relative links work
-    yesodDispatch _ _ _ _ _ _ [] _ req =
-        return $ responseLBS status301 [("Location", rawPathInfo req `S.append` "/")] ""
-
-    yesodDispatch _ (Static set) _ _ _ _ textPieces  _ req =
+    yesodDispatch _ _ (Static set) _ _ _ _ textPieces  _ req =
         staticApp set req { pathInfo = textPieces }
 
 notHidden :: Prelude.FilePath -> Bool
@@ -227,18 +221,18 @@ publicFiles :: Prelude.FilePath -> Q [Dec]
 publicFiles dir = mkStaticFiles' dir "StaticRoute" False
 
 
-mkHashMap :: Prelude.FilePath -> IO (M.Map FilePath S8.ByteString)
+mkHashMap :: Prelude.FilePath -> IO (M.Map F.FilePath S8.ByteString)
 mkHashMap dir = do
     fs <- getFileListPieces dir
     hashAlist fs >>= return . M.fromList
   where
-    hashAlist :: [[String]] -> IO [(FilePath, S8.ByteString)]
+    hashAlist :: [[String]] -> IO [(F.FilePath, S8.ByteString)]
     hashAlist fs = mapM hashPair fs
       where
-        hashPair :: [String] -> IO (FilePath, S8.ByteString)
+        hashPair :: [String] -> IO (F.FilePath, S8.ByteString)
         hashPair pieces = do let file = pathFromRawPieces dir pieces
                              h <- base64md5File file
-                             return (toFilePath file, S8.pack h)
+                             return (F.decodeString file, S8.pack h)
 
 pathFromRawPieces :: Prelude.FilePath -> [String] -> Prelude.FilePath
 pathFromRawPieces =
@@ -249,12 +243,12 @@ pathFromRawPieces =
 cachedETagLookupDevel :: Prelude.FilePath -> IO ETagLookup
 cachedETagLookupDevel dir = do
     etags <- mkHashMap dir
-    mtimeVar <- newIORef (M.empty :: M.Map FilePath EpochTime)
+    mtimeVar <- newIORef (M.empty :: M.Map F.FilePath EpochTime)
     return $ \f ->
       case M.lookup f etags of
         Nothing -> return Nothing
         Just checksum -> do
-          fs <- getFileStatus $ fromFilePath f
+          fs <- getFileStatus $ F.encodeString f
           let newt = modificationTime fs
           mtimes <- readIORef mtimeVar
           oldt <- case M.lookup f mtimes of
@@ -307,7 +301,7 @@ mkStaticFilesList fp fs routeConName makeHash = do
         pack' <- [|pack|]
         qs <- if makeHash
                     then do hash <- qRunIO $ base64md5File $ pathFromRawPieces fp f
-                            [|[(pack $(lift hash), mempty)]|]
+                            [|[(pack "etag", pack $(lift hash))]|]
                     else return $ ListE []
         return
             [ SigD routeName $ ConT route
@@ -337,35 +331,3 @@ base64 = map tr
     tr '+' = '-'
     tr '/' = '_'
     tr c   = c
-
-{- FIXME
--- | Dispatch static route for a subsite
---
--- Subsites with static routes can't (yet) define Static routes the same way "master" sites can.
--- Instead of a subsite route:
--- /static StaticR Static getStatic
--- Use a normal route:
--- /static/*Strings StaticR GET
---
--- Then, define getStaticR something like:
--- getStaticR = getStaticHandler ($(mkEmbedFiles "static") typeByExt) StaticR
--- */ end CPP comment
-getStaticHandler :: Static -> (StaticRoute -> Route sub) -> [String] -> GHandler sub y ChooseRep
-getStaticHandler static toSubR pieces = do
-  toMasterR <- getRouteToMaster
-  toMasterHandler (toMasterR . toSubR) toSub route handler
-  where route = StaticRoute pieces []
-        toSub _ = static
-        staticSite = getSubSite :: Site (Route Static) (String -> Maybe (GHandler Static y ChooseRep))
-        handler = fromMaybe notFound $ handleSite staticSite (error "Yesod.Static: getSTaticHandler") route "GET"
--}
-
-
-{-
-calcHash :: Prelude.FilePath -> IO String
-calcHash fname =
-    withBinaryFile fname ReadMode hashHandle
-  where
-    hashHandle h = do s <- L.hGetContents h
-                      return $! base64md5 s
-                      -}
